import praw
import traceback
from fast_utils import exact_word_match

r = praw.Reddit(client_id='jaFBw59_YwZq2g',
                client_secret='NX8gsx7NAppehlUJH3B3Db1yH7w',
                user_agent='Anotator', username='fastent_reddit',
                password='fastent2017')


def random_find_context_comments_depth(word):

    answer_list = []
    try:
        subreddit = r.subreddit('AskReddit')
        comment_iterator = 0
        for submission in subreddit.stream.submissions():
            submission.comments.replace_more(limit=None)
            kyanq = submission.comments.list()
            for comment in kyanq:

                comment_iterator += 1
                if comment_iterator >= 100:
                    return {word: answer_list}

                if exact_word_match(word, comment.body):
                    answer_list.append(comment.body)

                if len(answer_list) >= 2:
                    return {word: answer_list}

    except Exception as e:
        print(e)

    return {word: answer_list}


def radom_find_context_comments(word):

    answer_list = []
    try:
        subreddit = r.subreddit('AskReddit')
        comment_iterator = 0
        for submission in subreddit.stream.submissions():

            submission.comments.replace_more(limit=None)
            for comment in submission.comments:

                comment_iterator += 1
                if comment_iterator >= 100:
                    return {word: answer_list}

                if exact_word_match(word, comment.body):
                    answer_list.append(comment.body)

                if len(answer_list) >= 2:
                    return {word: answer_list}

    except Exception as e:
        print(e)

    return {word: answer_list}


def find_context_fast(word, min_context_amount=5):
    """
    Return a context for a word after fast title traverse

    Args:
        word (str): The word that needs context
        min_context_amount (int): maximum number of contexts to find_context_fast
    Returns:
         (dict) : {word (str): context (list)} the resulting pair of word:contexts
    """
    answer_list = []
    try:
        comment_iterator = 0
        for submission in r.subreddit('all').search(word):
            comment_iterator += 1
            if exact_word_match(word, submission.title):
                answer_list.append(submission.title)

            if len(answer_list) >= min_context_amount:
                return {word: answer_list}

    except Exception as e:
        print(traceback.format_exc())
        return None

    return {word: answer_list}


def find_context_long(word, min_context_amount=5, comment_depth=100):
    """
    Return a context for a word after long comment traverse
    
    Args:
        word (str): The word that needs context
        min_context_amount (int): maximum number of contexts to find_context_fast
        comment_depth (int): Maximum comment depth for traversal

    Returns:
         (dict) : {word (str): context (list)} the resulting pair of word:contexts
    """

    answer_list = []
    try:
        for submission in r.subreddit('all').search(word):
            comment_iterator = 0
            if len(answer_list) >= min_context_amount:
                return {word: answer_list}

            if exact_word_match(word, submission.title):
                    answer_list.append(submission.title)

            submission.comments.replace_more(limit=0)
            for comment in submission.comments:
                comment_iterator += 1
                if comment_iterator >= comment_depth:
                    break  #dont wont to use break really

                if exact_word_match(word, comment.body):
                    print("good comment")
                    answer_list.append(comment.body)

                if len(answer_list) >= min_context_amount:
                    return {word: answer_list}

    except Exception as e:
        print(traceback.format_exc())

    return {word: answer_list}



import os
import time
from itertools import groupby
import re
import traceback



def log_to_text(full_metadata_list, file_name):
	try:
		if not (os.path.isfile(file_name + ".txt")):
			file=open(file_name + ".txt", "w+", encoding='utf-8', errors="surrogateescape")
			append_string = str(full_metadata_list)
			file.write(append_string)
			file.close()
		else:
			file=open(file_name + ".txt", "a+", encoding='utf-8', errors="surrogateescape")
			append_string = str(full_metadata_list)
			file.write(append_string)

	except IOError as e:
		print(e)
		print(1)

def getopts(argv):
	opts = {}  # Empty dictionary to store key-value pairs.
	while argv:  # While there are arguments left to parse...
		if argv[0][0] == '-':  # Found a "-name value" pair.
			opts[argv[0]] = argv[1]  # Add key and value to the dictionary.
		argv = argv[1:]  # Reduce the argument list by copying it starting from index 1.

	return opts

def remove_special_symbols(value):
	new_value =[]
	keep_char_list = ['@','.',',','!']
	for string in value:
		new_string = ''.join(e for e in string if (e.isalnum() or e in [x for x in keep_char_list]))
		if new_string:
			new_value.append(new_string)

	return new_value


def flatten(chunkList):
	sentences_split = []
	for chunk in chunkList:
		for word in chunk:
			sentences_split.append(word)
	return sentences_split


def split_with_indices(s, c=' '):
	p = 0
	for k, g in groupby(s, lambda x:x==c):
		q = p + sum(1 for i in g)
		if not k:
			yield p, q # or p, q-1 if you are really sure you want that
		p = q

def list_segmentor(seq, size):
	newseq = []
	splitsize = 1.0/max(1,size)*len(seq)
	for i in range(size):
			newseq.append(seq[int(round(i*splitsize)):int(round((i+1)*splitsize))])
	return newseq

def exact_word_match(word, raw_sentence):
	lister = []
	try:
		regexp_pattern = r"(?:^|\W)" + word + r"(?:$|\W)"
		#regexp_verify = re.compile(regexp_pattern) #no Need to save
		lister = re.findall(regexp_pattern, raw_sentence, flags=re.IGNORECASE)
	except Exception as e:
		print(str(traceback.format_exc()))
		return False
	return len(lister)>=1



def replace_all(text, dic):
	for i, j in dic.items():
		text = text.replace(i, j)
	return text



from text_utils import stop_word_remove
from text_utils import adverb_remove
from text_utils import verb_remove
from text_utils import adjective_remove
from text_utils import special_symbols_remove
from fast_utils import split_with_indices
import settings
import argparse
import time

def dataset_NER_prepocess(dataset):
    """
    Preprocess a dataset before training NER.

    Assuming That a clean dataset of Entities should not contain
    verbs, adverbs, adjectives and random symbols

    Args:
        dataset (list): list of strings for NER trainging
    Returns:
        list: processed dataset if Sucessful, None otherwise
    """
    preprocessed = []

    try:
        preprocessed = stop_word_remove(dataset)

        if not preprocessed:
            preprocessed = dataset

        preprocessed = adverb_remove(dataset)

        if not preprocessed:
            preprocessed = dataset

        preprocessed = verb_remove(dataset)

        if not preprocessed:
            preprocessed = dataset
        preprocessed = adjective_remove(dataset)

        if not preprocessed:
            preprocessed = dataset

        preprocessed = special_symbols_remove(dataset)

    except Exception as e:
        print(e)
        return None

    return preprocessed


def dataset_to_spacy(db, entity_label):
    """
    Bring a dataset to a spacy trainable state

    Args:
        dataset (list): list of strings for NER trainging
        entity_label (str): designated label for the Entity
    Returns:
        list: The spacy training ready list if Sucessful, None otherwise
    """

    train_data = []
    try:
        #anyone missing context ??
        for word in db[entity_label]:
            if not db[entity_label][word]['context']:
                continue

            for contexted_example in db[entity_label][word]['context']:
                entities = []

                if len(word.split(" ")) > 1 :
                    start = contexted_example.lower().find(word.lower())
                    end = start+len(word)
                    entities.append((start, end, entity_label))

                else:
                    splits = list(split_with_indices(contexted_example))
                    for touple in splits:
                        if word.lower() in contexted_example[touple[0]:touple[1]].lower():
                            entities.append((touple[0], touple[0] + len(word), entity_label))

                train_data.append((contexted_example, {'entities': entities}))

    except Exception as e:
        print(e)
        return None

    return train_data


if __name__ == '__main__':

    settings.init()

    parser = argparse.ArgumentParser(description='Anotator options')
    entity_label = 'random_ent_' + str(time.time)
    parser.add_argument('-e', action="store", type=str, dest = 'entity_label', help ='designated entity label', default=entity_label)
    entity_label = parser.entity_label

    couchdb = settings.couchdb

    dataset_to_spacy(couchdb, entity_label)



import argparse
import logging
from wordent_utils import word_to_wn
from gensim.models.poincare import PoincareModel, PoincareKeyedVectors, PoincareRelations

logging.basicConfig(level=logging.INFO)


def poincare_train(hypertouple_dataset, size=2, burn_in=0, epochs = 5, print_freq = 100):
    """
    Train a poincare embedding

    Args:
        hypertouple_dataset (list): The hypertouple dataset to feed for training
        size (int): size of model
        burn_in (int): Burnin identifier
        epochs (int): Number of epochs to train
        print_freq (int): Update frequency number

    Returns:
        poincare_model (model object) : The trained Poincare Model
    """
    poincare_model = None
    try:

        #poincare_model = PoincareModel(train_data = hypertouple_dataset)
        poincare_model = PoincareModel(train_data=hypertouple_dataset, size = size, burn_in = burn_in)
        poincare_model.train(epochs=epochs, print_every = print_freq)

    except Exception as e:
        print(e)
    return poincare_model



def poincare_simmilar(poincare_model, word):
    """
    Return the list of words closest to word

    Args:
        poincare_model (model object): The trained poincare model to use
        word (str): The word used for finding similar words list

    Returns:
        most_simmilar_set (list) : The list of similar words
    """

    most_simmilar_set = None
    try:
        wnet_word = word_to_wn(word)
        most_simmilar_set = poincare_model.kv.most_similar(wnet_word)
    except Exception as e:
        print(e)

    return most_simmilar_set


def poincare_closer_then(poincare_model, word1, word2):
    """
    Return the list of words closer to word1 in comparison with word2

    Args:
        word1 (str): first word
        word2 (str): second word

    Returns:
        closer_list (list) : The list of segmented words
    """


    closer_list = None
    try:
        wnet_word1 = word_to_wn(word1)
        wnet_word2 = word_to_wn(word2)

        closer_list = poincare_model.kv.closer_than(wnet_word1, wnet_word2)
    except Exception as e :
        print(e)

    return closer_list


def poincare_word_dist(poincare_model, word1, word2):
    """
    Return the distance of words between word1 and word2

    Args:
        word1 (str): first word
        word2 (str): second word

    Returns:
        dist (float) : The list of segmented words
    """


    dist = None
    try:
        wnet_word1 = word_to_wn(word1)
        wnet_word2 = word_to_wn(word2)
        dist = poincare_model.kv.distance(wnet_word1, wnet_word2)
    except Exception as e :
        print(e)

    return dist


def poincare_closest_child(poincare_model, word):
    """
    Return the closet child node for a given word

    Args:
        word (str): arbitrary word

    Returns:
        child_word (str) : The closest child word in Wordnet format
    """

    child_word = None
    try:
        wnet_word = word_to_wn(word)
        child_word = poincare_model.kv.closest_child(wnet_word)
    except Exception as e :
        print(e)

    return child_word


def poincare_closest_parent(poincare_model, word):
    """
    Return the closet parent node for a given word
    
    Args:
        word (str): arbitrary word

    Returns:
        child_word (str) : The closest parent word in Wordnet format
    """

    parent_word = None
    try:
        wnet_word = word_to_wn(word)
        parent_word = poincare_model.kv.closest_parent(wnet_word)
    except Exception as e :
        print(e)

    return parent_word



if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='Anotator options')
    parser.add_argument('-p', action="store", type=str, dest = 'dataset_path', help ='designated dataset path', default = None)
    parser.add_argument('-d', action="store", type=str, dest = 'delimiter', help ='the split for the hyper touples', default = '\t')

    path = parser.path

    dataset = PoincareRelations(file_path=path, delimiter='\t')

    model = poincare_train(dataset)



from __future__ import unicode_literals, print_function
import os
import random
from pathlib import Path
import spacy


# training data
# Note: If you're using an existing model, make sure to mix in examples of
# other entity types that spaCy correctly recognized before. Otherwise, your
# model might learn the new type, but "forget" what it previously knew.
# https://explosion.ai/blog/pseudo-rehearsal-catastrophic-forgetting


def pseudo_rehersal(model):
    #TODO add the rehersal
    return 0


def train(dataset=[], model=None, new_model_name=None, output_dir=os.getcwd()+"/models", n_iter=10, Label="random_ent", verbose=False):
    """Set up the pipeline and entity recognizer, and train the new entity."""

    if not dataset:
        return None

    if model is not None:
        nlp = spacy.load(model)  # load existing spaCy model

        if verbose:
            print("Loaded model '%s'" % model)
    else:
        nlp = spacy.blank('en')  # create blank Language class

        if verbose:
            print("Created blank 'en' model")

    # Add entity recognizer to model if it's not in the pipeline
    # nlp.create_pipe works for built-ins that are registered with spaCy
    if 'ner' not in nlp.pipe_names:
        ner = nlp.create_pipe('ner')
        nlp.add_pipe(ner)
    # otherwise, get it, so we can add labels to it
    else:
        ner = nlp.get_pipe('ner')

    ner.add_label(Label)   # add new entity label to entity recognizer

    # get names of other pipes to disable them during training
    other_pipes = [pipe for pipe in nlp.pipe_names if pipe != 'ner']

    with nlp.disable_pipes(*other_pipes):  # only train NER
        optimizer = nlp.begin_training()
        for itn in range(n_iter):
            random.shuffle(dataset)
            losses = {}
            for text, annotations in dataset:
                nlp.update([text], [annotations], sgd = optimizer, drop=0.35,losses=losses)
            print(losses)

    '''
    # test the trained model
    test_text = 'Do you like MIT?'
    doc = nlp(test_text)
    print("Entities in '%s'" % test_text)
    for ent in doc.ents:
        print(ent.label_, ent.text)
    '''

    # save model to output directory
    if output_dir is not None:
        output_dir = Path(output_dir)
        if not output_dir.exists():
            output_dir.mkdir()
        nlp.meta['name'] = new_model_name  # rename model
        nlp.to_disk(output_dir)
        print("Saved model to", output_dir)

        '''
        # test the saved model
        print("Loading from", output_dir)
        nlp2 = spacy.load(output_dir)
        doc2 = nlp2(test_text)
        for ent in doc2.ents:
            print(ent.label_, ent.text)
        '''

        return nlp



from nltk.corpus import wordnet as wn
from nltk.stem import WordNetLemmatizer
from nltk import pos_tag, word_tokenize
from nltk.corpus import wordnet as wn
from fast_utils import replace_all
import traceback

def penn_to_wn(tag):
    """
    Convert PENN to WordNet Format
    
    Args:
        tag (str): PENN Tag

    Returns:
        str: Wordent Representation
    """

    try:

        if tag.startswith('J'):
            return wn.ADJ
        elif tag.startswith('N'):
            return wn.NOUN
        elif tag.startswith('R'):
            return wn.ADV
        elif tag.startswith('V'):
            return wn.VERB

    except Exception as e:
        print(e)

    return None


def word_to_wn(word):
    """
    Convert a string/word to WordNet Format

    Args:
        word (str): an arbitrary word

    Returns:
        str: Wordent Representation
    """

    ans = None
    try:

        tagged = pos_tag(word_tokenize(word))
        synsets = []
        for token in tagged:
            wn_tag = penn_to_wn(token[1])
            if not wn_tag:
                continue

            synsets.append(wn.synsets(token[0], pos=wn_tag)[0])

        reps = {'Synset':'', '(':'', ')':'','\'':''}
        ans = replace_all(str(synsets[0]),reps)

    except Exception as e:
        print(e)
        return None

    return ans



def word_to_hyperTouple(word):

    """
    Given an arbitrary word, return a touple of a word and the closest ancestor hypernym in the hierarchy

    Args:
        word (str): an arbitrary word

    Returns:
        str: Wordent Representation
    """

    ans = None
    try:

        wn_word = word_to_wn(word)
        base_hypernym_closure = wn.synset(wn_word).closure(lambda s:s.hypernyms())
        hypernym_list= set([i for i in base_hypernym_closure])

        #depth calc
        max_depth = max(s.min_depth() for s in hypernym_list)
        base_hypernym_synset= [s for s in hypernym_list if s.min_depth() == max_depth]
        print(base_hypernym_synset)
        if base_hypernym_synset:
            string_hyper= str(base_hypernym_synset[0])
            reps = {'Synset':'', '(':'', ')':'','\'':''}
            base_hypernym = replace_all(string_hyper,reps)
            ans = (wn_word, base_hypernym)

    except Exception as e:
        print(e)
        return None

    return ans

def wordnet_synonyms(word):
    """
    Given an arbitrary word, return the closest WordNet synonym list

    Args:
        word (str): an arbitrary word

    Returns:
        synonyms (list): synonym list
    """
    synonyms = None
    try:
        for syn in wn.synsets(word):
            for l in syn.lemmas():
                synonyms.append(l.name())

    except Exception as e:
        print(e)
        return None

    return synonyms



def wordnet_context(word):
    context = None

    try:
        word_Wnet = word_to_wn(word)

        if not word_Wnet:
            return None

        context = wn.synset(word_Wnet).examples()

    except Exception as e:
        print(traceback.format_exc())
        return None

    return {word:context}



from nltk.corpus import stopwords
from nltk.tokenize import sent_tokenize, word_tokenize
from nltk import pos_tag
from fast_utils import flatten
from fuzzywuzzy import fuzz

stop_words_cache = stopwords.words("english")



def fuzzy_word_remove(word_list):
    """
    Remove words that are alike from the list

    Args:
        word_list (str): Arbitrary list of words

    Returns:
        list: The cleaned list
    """
    try:
        temp = word_list
        for i in range(len(word_list) -1):
            for j in range(len(word_list) -1):
                if i != j:
                    if (fuzz.token_sort_ratio(temp[i], temp[j]) > 75 ):
                        temp.remove(temp[j])
    except Exception as e:
        print(e)

    return temp

def text_segmentator(full_text):
    """
    Segment the text on word by word basis, while removing stopwords

    Args:
        full_text_list (list): Text split in several lists

    Returns:
        list: The list of segmented words
    """

    try:
        if not full_text:
            return None

        if all(x in [None, ''] for x in full_text):
            return None

        else:
            sentences = stop_word_remove(full_text)
            sentences = sent_tokenize(sentences)
            sentences = [word_tokenize(sent) for sent in sentences]
            sentences = flatten(sentences)

            #REMOVING NEW STEP
            sentences = special_symbols_remove(sentences)

    except Exception as e:
        print("text_segmentator error: "+ str(e))
        return None

    return sentences



def stop_word_remove(full_text_list):
    """
    Remove stopwords from a list of texts and return the list

    Args:
        full_text_list (str): Text split in several lists

    Returns:
        list: The initial list of strings with no stopwords if successful, None if failed
    """

    try:
        full_list = []
        for text in full_text_list:
            if text:
                full_text = " ".join([word for word in text.split() if word not in stop_words_cache])
                full_text = full_text.strip()
                full_list.append(full_text)

    except Exception as e:
        print(e)
        return None

    return full_text


def adverb_remove(full_text_list):
    """
    Remove adverbs from a list of texts and return the list

    Args:
        full_text_list (str): Text split in several lists

    Returns:
        list: The initial list of strings with no adverbs, if successful, None if failed
    """

    try:
        tag_list = "RB"
        full_list = []
        for text in full_text_list:
            if text:
                full_text = " ".join([word for word in text.split() if (tag_list not in pos_tag([word])[0][1])])
                full_text = full_text.strip()
                full_list.append(full_text)

    except Exception as e:
        print(e)
        return None

    return full_list


def verb_remove(full_text_list):
    """
    Remove verbs from a list of texts and return the list

    Args:
        full_text_list (str): Text split in several lists

    Returns:
        str: The initial list of strings with no verbs, if successful, None if failed
    """

    try:
        tag_list = "VB"
        full_list = []
        for text in full_text_list:
            if text:
                full_text = " ".join([word for word in text.split() if (tag_list not in pos_tag([word])[0][1])])
                full_text = full_text.strip()
                full_list.append(full_text)

    except Exception as e:
        print(e)
        return None

    return full_list


def adjective_remove(full_text_list):
    """
    Remove adjectives from a list of texts and return the whole list

    Args:
        full_text_list (str): Text split in several lists

    Returns:
        str: The initial list of strings with no adjectives, if successful, None if failed
    """

    try:
        tag_list = "JJ"
        full_list = []
        for text in full_text_list:
            if text:
                full_text = " ".join([word for word in text.split() if (tag_list not in pos_tag([word])[0][1])])
                full_text = full_text.strip()
                full_list.append(full_text)

    except Exception as e:
        print(e)
        return None

    return full_list

def special_symbols_remove(full_text_list):
    """
    Remove special symbols from a list of texts and return the list. (Few relevant symbols stay)

    Args:
        full_text_list (str): Text split in several lists

    Returns:
        list: The initial list of strings with no adjectives, if successful, None if failed
    """

    try:
        new_value =[]
        keep_char_list = ['-','.',','] #SUBJECT TO CHANGE
        for text in full_text_list:
            new_string = ''.join(e for e in text if (e.isalnum() or e in [x for x in keep_char_list]))
            if new_string:
                new_value.append(new_string)

    except Exception as e:
        print(e)

    return new_value



import subprocess
import sys
import argparse
import requests
import shutil
import lxml.html as LH
import pandas as pd
import urllib.request
from fast_utils import exact_word_match


class DownloadError(Exception):
    def __init__(self, output):
        self.output = output

def spacy_model_download(model_name, timeout = None):
    """
    Downloads a spacy model with name
    
    Args:
        model_name (str): The model name for download
    Returns:
         (void) : download in the designated folder of fastent
    """
    try :

        if sys.version_info <=(3,4):

            arguments = [python_exec, "-m",'spacy','download',model_name]
            subprocess.call(arguments, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

            # process = subprocess.call(arguments, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            #
            # output_cont = process.stdout.decode("ISO-8859-1", "ignore")
            #
            # if not exact_word_match('Successfully',output_cont):
            #     raise DownloadError(process.stdout.decode("ISO-8859-1", "ignore"))
            # else:
            #     return filename.group(1)

        else:

            arguments = [python_exec(), '-m','spacy','download', model_name]
            print("Dowload for model {} stared".format(model_name))
            process = subprocess.run(arguments, stdout=subprocess.PIPE, stderr=subprocess.PIPE, timeout=timeout)
            output_cont = process.stdout.decode("ISO-8859-1", "ignore")
            print("Dowload for model {} ended".format(model_name))

            if not exact_word_match('Successfully',output_cont):
                raise DownloadError(process.stdout.decode("ISO-8859-1", "ignore"))
            else:
                return output_cont

    except (DownloadError, Exception) as e:
        print(e)


def fasttext_list():
    """
    Return a Dictionary of the possible fasttext models

    Args:
        None:
    Returns:
         diction_frac(dict) : Language to Model dictionary
    """

    diction_frac = {}
    try:
        content = requests.get("https://github.com/facebookresearch/fastText/blob/master/pretrained-vectors.md").content
        webpage = LH.fromstring(content)
        allRefs = webpage.xpath('//a/@href')

        allRefs = [i for i in allRefs if 'amazonaws' in i and not 'zip' in i]
        allRefs

        df = pd.read_html(content)
        df = df[-1]

        assert(len(allRefs) ==  len(df['Unnamed: 0']) + len(df['Unnamed: 1'])+len(df['Unnamed: 2']))

        for i in range(len(allRefs)):
            if i%3 == 0:
                diction_frac[df['Unnamed: 0'][int(i/3)]] = allRefs[i]
            if i%3 == 1:
                diction_frac[df['Unnamed: 1'][int(i/3)]] = allRefs[i]
            if i%3 == 2:
                diction_frac[df['Unnamed: 2'][int(i/3)]] = allRefs[i]


    except Exception as e:
        print(e)
        return None

    return diction_frac

def fasttext_dowload(language_name, timeout = None):
    """
    Downloads a fasttext model with language name

    Args:
        language_name (str): The language name for download
    Returns:
         (void) : download in the designated language model to fastent folder
    """
    try:
        full_lang_dict = fasttext_list()
        url = ''
        for key in full_lang_dict:
            if language_name.lower() in key.lower():
                url = full_lang_dict[key]
                file_name = url.split('/')[-1]

        with urllib.request.urlopen(url) as response, open(file_name, 'wb') as out_file:
            shutil.copyfileobj(response, out_file)

    except Exception as e:
        print(e)



def python_exec():
    if sys.version_info <(3,):
        return 'python'

    return 'python3'


if __name__ == "__main__":

    parser = argparse.ArgumentParser(description='API options')
    parser.add_argument('-l', action="store", dest = 'location', help = 'Location of the model, i.e gensim, spacy, fastText etc etc')
    parser.add_argument('-m', action="store", type=str, dest = 'model_name', help ='designated model name')
    parser.add_argument('-t', action="store", type=str, dest = 'timeout', help ='timeout', default = None)

    results = parser.parse_args()
    print(results)

    if 'spacy' in results.location.lower():
        spacy_model_download(results.model_name, results.timeout)
    if 'fasttext' in results.location.lower():
        fasttext_dowload(results.model_name, results.timeout)



import couchdb


def init(username = None, password = None):
    """
    Global Initialization of settings

    Return the list of words closer to word1 in comparison with word2
    Args:
        word1 (str): first word
        word2 (str): second word

    Returns:
        void: As it is global settings init nothing is returned
    """

    global couchDB

    if not username:
        couchDB = couchdb.Server("http://127.0.0.1:5984/")
    else:
        couchDB = couchdb.Server("http://%s:%s@127.0.0.1:5984/" % (username, password))

def db_initialize(dbname):
    """
    Return of single db from cauchDB server if present or creates if not present

    Args:
        dbname (str): The designated database name

    Returns:
        db (Database object): the created or retrieved database
    """
    try:
        if dbname in couchDB:
            db = couchDB[dbname]
        else:
            db = couchDB.create(dbname)

    except Exception as e:
        print(e)
        return None

    return db



import time
import multiprocessing
import traceback
import settings
from fast_utils import list_segmentor
from reddit_utils import find_context_fast
from reddit_utils import find_context_long
from wordent_utils import wordnet_context



def contextualize(word = 'none', option = 'fast', dbname = str(time.time())):
    """
    Gettting a context with a speified method and saving in DB

    Args:
        word (str): The word meant for contextualization
        option (str): Type of contextualization we aim for
        dbname (str): The dbname that we save in
    Returns:
         context_dict (dict) : The strucutred dictionary for word and contexts
    """
    try:

        db = settings.couchDB[dbname]

        if word in db:
            return None

#        wNet_dict = wordnet_context(word)
        context_dict = {'context':[],'_id': word}
        if 'fast' in option:
            context = find_context_fast(word)
        if 'long' in option:
            context = find_context_long(word)

        if context:
            context_dict['context'] = context[word]

        #if wNet_dict and context:
         #   context_dict['context'] = wNet_dict[word] + context[word]


        db.save(context_dict)

    except Exception as e:
        print(traceback.format_exc())
        return None

    return context_dict



def list_contextualize(proc_list= [], option = 'fast', iterator = 0, dbname = str(time.time())):
    """
    Gettting a context with a speified method for the given word list

    Args:
        proc_list (list): The word list meant for contextualization
        option (str): Type of contextualization we aim for
        iterator (int): Processing Thread number
        dbname (str): The dbname that we save in
    Returns:
         (void)
    """
    for word in proc_list[iterator]:
        _ = contextualize(word, option, dbname)




def parallel_runner(process_number, proc_list, option, dbname):
    """
    Initialize DB for words. Running Parallel threads for contextualization.

    Args:
        proc_list (list): The word list meant for contextualization
        option (str): Type of contextualization we aim for
        process_number (int): Processing Thread number
        dbname (str): The dbname that we save in
    Returns:
         (void)
    """
    settings.init()
    # Run tasks using processes
    segmented_list = list_segmentor(proc_list, process_number)

    print(segmented_list)
    processes = [multiprocessing.Process(target = list_contextualize, args = ([segmented_list, option, iterator, dbname])) for iterator in range(process_number)]
    [process.start() for process in processes]
    [process.join() for process in processes]



import spacy
import gensim
import traceback
import argparse
from text_utils import fuzzy_word_remove
from fast_utils import log_to_text

def spacy_initialize(model_name):
    """
    Initialize a spacy model with name

    Args:
        model_name (str): The designated model name
    Returns:
        model (spacy model object): spacy model
    """
    try:
        model = spacy.load(model_name)
    except Exception as e:
        print(e)
        return None

    return model


def gensim_initialize(model_name):
    #todo
    try:

        model = spacy.load('en_core_web_lg')
    except Exception as e:
        print(e)
        return None

    return model


def similar_set_spacy(model, word_list, max_similar_amount=100):
    """
    Return the List of similar words using the spacy model and suggested word_list

    Args:
        model (spacy model object): The spacy model
        word_list (list): The list of suggestive words
        max_similar_amount (int): The maximum number of words to look for

    Returns:
        dataset (list): list of similar words
    """
    try:

        similarities = []
        for base_word in word_list:
            word = model.vocab[base_word]
            possible_words = [w for w in word.vocab if w.is_lower == word.is_lower and w.prob >= -20 and not w.is_stop and len(w.text) >1]
            #remove alpha numeral suspicious cases
            #possible_words = set([p for p in possible_words if p.text.isalnum()])
            by_similarity = sorted(possible_words, key=lambda w: word.similarity(w), reverse=True)
            similarities.append(by_similarity)


        '''
        overlaps = None
        for sim_set in similarities:
            if not overlaps:
                overlaps  = sim_set
            else:
                overlaps = overlaps.intersection(sim_set)

        overlaps = list(overlaps)
        while len(overlaps) < max_similar_amount:
            possible_top = random.randint(1, 10)
            possib_addition = similarities[random.randint(0, len(similarities)-1)][possible_top]
            if (possib_addition not in overlaps):
                overlaps.append(possib_addition)
        '''

        overlaps = []
        i = 1
        while len(overlaps) < max_similar_amount:
            for j in range(len(similarities) - 1):
                overlaps +=  [w.lower_ for w in (similarities[j][10*(i-1):10*i])]

            overlaps = list(set(overlaps))
            i += 1

        overlaps = fuzzy_word_remove(overlaps)

        while len(overlaps) < max_similar_amount:
            for j in range(len(similarities) - 1):
                overlaps +=  [w.lower_ for w in (similarities[j][10*(i-1):10*i])]

            overlaps = list(set(overlaps))
            i += 1

        overlaps = fuzzy_word_remove(overlaps)

    except RuntimeError as e:
        print(traceback.format_exc())

    dataset = overlaps
    dataset = dataset + word_list

    return dataset


def dataset_generate(model_name = 'en_core_web_sm',suggestions = [], max_similar_amount = 100):

    try:
        model = spacy_initialize(model_name)
        suggestions = []
        for sug in results.suggestions.split(","):
            suggestions.append(sug.strip())

        similarity_set = similar_set_spacy(model,suggestions, max_similar_amount)

        log_to_text(', '.join(similarity_set), "raw_data")

    except Exception as e:
        print(e)



if __name__ == "__main__":

    parser = argparse.ArgumentParser(description='API options')
    parser.add_argument('-m', action="store", type=str, dest = 'model_name', help ='designated model name')
    parser.add_argument('-s', action="store", type=str, dest = 'suggestions', help ='entity word sugesstions')
    parser.add_argument('-max', action="store", type=int, dest = 'max_similar_amount', help ='maximum size of resulting dataset')

    results = parser.parse_args()

    if results.max_similar_amount is not None:
        dataset_generate(results.model_name, results.suggestions,results.max_similar_amount )
    else:
        dataset_generate(results.model_name, results.suggestions)



from setuptools import setup
#from distutils.core import setup
import os
import subprocess

#with open('fastent/requirements.txt') as f:
 #   requirements = f.read().splitlines()

#print(requirements)
def readme():
    with open('README.rst') as f:
        return f.read()

setup(
  name = 'fastent',
  packages = ['fastent'], # this must be the same as the name above
  version = '0.5.8.7',
  description = 'Automated Custom NER tool',
  author = 'Erik Arakelyan',
  author_email = 'erikarakelyan1997@gmail.com',
  scripts=['fastent/fastent_install.sh'],
  install_requires=['alabaster==0.7.10', 'argh==0.26.2', 'Babel==2.5.3', 'boto==2.48.0', 'boto3==1.7.24', 'botocore==1.10.24', 'bz2file==0.98', 'certifi==2018.4.16', 'chardet==3.0.4', 'CouchDB==1.2', 'cymem==1.31.2', 'cytoolz==0.8.2', 'dill==0.2.7.1', 'docutils==0.14', 'fuzzywuzzy==0.16.0', 'gensim==3.4.0', 'idna==2.6', 'imagesize==1.0.0', 'Jinja2==2.10', 'jmespath==0.9.3', 'livereload==2.5.2', 'lxml==4.2.1', 'MarkupSafe==1.0', 'msgpack-numpy==0.4.1', 'msgpack-python==0.5.6', 'murmurhash==0.28.0', 'nltk==3.3', 'numpy==1.14.3', 'packaging==17.1', 'pandas==0.23.0', 'pathlib==1.0.1', 'pathtools==0.1.2', 'plac==0.9.6', 'port-for==0.3.1', 'praw==5.4.0', 'prawcore==0.14.0', 'preshed==1.0.0', 'Pygments==2.2.0', 'pyparsing==2.2.0', 'python-dateutil==2.7.3', 'python-Levenshtein==0.12.0', 'pytz==2018.4', 'PyYAML==3.12', 'regex==2017.4.5', 'requests==2.18.4', 's3transfer==0.1.13', 'scipy==1.1.0', 'six==1.11.0', 'smart-open==1.5.7', 'snowballstemmer==1.2.1', 'spacy==2.0.11', 'Sphinx==1.7.4', 'sphinx-autobuild==0.7.1', 'sphinxcontrib-websupport==1.0.1', 'termcolor==1.1.0', 'thinc==6.10.2', 'toolz==0.9.0', 'tornado==5.0.2', 'tqdm==4.23.3', 'ujson==1.35', 'update-checker==0.16', 'urllib3==1.22', 'watchdog==0.8.3', 'wrapt==1.10.11']
,
  url = 'https://github.com/fastent/fastent', # use the URL to the github repo
  long_description=readme(),
  download_url = 'https://github.com/fastent/fastent/archive/0.5.tar.gz', # I'll explain this in a second
  keywords = ['NER', 'Anotation', 'Contextualization'], # arbitrary keywords
  classifiers = [],
)


